# -------------------------------------------------------------------------
#
#        D E S T I N A T I O N   S Y S T E M
#
# -------------------------------------------------------------------------
from .destinations.interfaces import *
from .destinations.tar import TarArchiver
from .destinations.local import LocalDestination
from .destinations.checksum import ChecksumDecorator
from .destinations.sharedweb import SharedWebDestination
from .destinations.fetchfile import FetchFileGenerator


# class SubdirectoryDecorator(DestinationDecorator):
#     def __init__(self, underlying: Destination, subdirectory: str):
#         super().__init__(underlying)
#         self.subdirectory = subdirectory
#
#     def add_file(self, relative_path: str, file: pathlib.Path):
#         self.underlying.add_file(self.subdirectory + "/" + relative_path, file)


class DestinationBuilder:
    """
    To facilitate building a stack of destination and its decorators.
    """

    def __init__(self, context: DeliveryContextIF):
        self.context = context
        self._destination = None

    def local(self, path: pathlib.Path):
        """Add a local destination with the given path"""
        self._destination = LocalDestination(self.context, path)
        return self

    def web(self):
        self._destination = SharedWebDestination(self.context)
        return self

    def checksums(self):
        self._destination = ChecksumDecorator(self._destination)
        return self

    def curlfile(self):
        self._destination = FetchFileGenerator(self._destination)
        return self

    def tar(self):
        """Add the tar decorator to the destination"""
        self._destination = TarArchiver(self.context, self._destination)
        return self

    def build(self):
        """Create the destination"""
        return self._destination
