import filecmp
import pathlib
from os import path

import pytest

from delivery.deliverer import LocalDestination


@pytest.fixture
def dest_dir(tmpdir) -> pathlib.Path:
    # generate the destination directory
    dest_dir = pathlib.Path(tmpdir) / "after"
    dest_dir.mkdir()
    return dest_dir


@pytest.fixture
def file_to_deliver(tmpdir) -> pathlib.Path:
    # rewrap the tmpdir as a pathlib.Path instance
    tmpdir = pathlib.Path(tmpdir)

    # generate the test file in prior
    (tmpdir / "prior").mkdir()
    file_name = "test.txt"

    temp_file = tmpdir / "prior" / file_name  # place a file into the source
    temp_file.write_text("content", encoding=None)  # write something into that file

    # secretly generate a second file
    temp_file2 = tmpdir / "prior" / "test2.txt"  # make another file in source
    temp_file2.write_text("more content", encoding=None)  # add some content to the second file

    return temp_file


def test_local_delivery_add_file(tmpdir, file_to_deliver: pathlib.Path, dest_dir: pathlib.Path):
    """
    Ensure that local delivery works
    """
    # generate the destination
    local_dest = LocalDestination(None, dest_dir)

    # add the file to it
    local_dest.add_file(file_to_deliver, file_to_deliver.name)

    # see if the source file eneded up in the destination
    assert (dest_dir / file_to_deliver.name).exists()

    # see if the content of the file is intact
    assert (dest_dir / file_to_deliver.name).read_text(encoding=None) == "content"


def test_local_delivery_add_directory(
    tmpdir: pathlib.Path, file_to_deliver: pathlib.Path, dest_dir: pathlib.Path
):
    """Ensure that local delivery does something"""
    tmpdir = pathlib.Path(tmpdir)

    local_dest = LocalDestination(None, dest_dir)
    local_dest.add_directory(file_to_deliver.parent, file_to_deliver.parent.name)

    compare_dirs = filecmp.dircmp(file_to_deliver.parent, dest_dir / "prior")
    # see if the destination got all the files from source
    assert (
        len(compare_dirs.left_only) == 0
        and len(compare_dirs.right_only) == 0
        and len(compare_dirs.funny_files) == 0
    )
