# coding: utf-8
from sqlalchemy import Column, ForeignKey, Integer, String
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship

Base = declarative_base()
metadata = Base.metadata


class NGASHost(Base):
    __tablename__ = 'ngas_hosts'
    host_id = Column('host_id', String, primary_key=True)
    domain = Column('domain', String)
    port = Column('srv_port', Integer)
    cluster_name = Column('cluster_name', String)
    disks = relationship('NGASDisk', backref='host')

    @property
    def server(self):
        return f'{self.host_id}.{self.domain}'

    def __repr__(self):
        return f'<NGASHost {self.host_id}.{self.domain}:{self.port}>'


class NGASDisk(Base):
    __tablename__ = 'ngas_disks'
    disk_id = Column('disk_id', String, primary_key=True)
    mounted = Column('mounted', Integer)
    host_id = Column('host_id', ForeignKey('ngas_hosts.host_id'))
    files = relationship('NGASFile', backref='disk')

    def __repr__(self):
        return f'<NGASDisk#{self.disk_id} mounted={self.mounted}>'


class NGASFile(Base):
    __tablename__ = 'ngas_files'
    file_id = Column('file_id', String, primary_key=True)
    file_name = Column('file_name', String)
    ingestion_date = Column('ingestion_date', String)
    file_size = Column('file_size', Integer)
    version = Column('file_version', Integer, primary_key=True)
    disk_id = Column('disk_id', ForeignKey('ngas_disks.disk_id'))
    checksum = Column('checksum', String)
    checksum_plugin = Column('checksum_plugin', String)
    format = Column('format', String)

    def __repr__(self):
        return f'<NGASFile#{self.file_id} name={self.file_name} size={self.file_size} version={self.version}>'
