import json
import os
from typing import List

import pytest

from workspaces.workflow.schema import WorkflowRequest, WorkflowRequestFile
from workspaces.workflow.services.workflow_info import WorkflowInfo
from workspaces.workflow.services.workflow_service import WorkflowService

pytest_plugins = ["testing.utils.conftest"]

"""
Tests for WorkflowService
"""


@pytest.mark.usefixtures("mock_workflow_info", "mock_workflow_service", "test_router")
class TestWorkflowService:
    requests = []

    def test_processing_site(self, mock_workflow_service: WorkflowService):
        wfs = mock_workflow_service
        assert isinstance(wfs.useCondor, bool) is True
        # assert wfs.useCondor is False

    @pytest.mark.skip(
        "Irreparably broken. Tries to use uninitialized fields of WorkflowRequest."
        "Probably safe to ignore as this is also tested in test_workflow_info.py"
    )
    def test_create_workflow_request(self, mock_workflow_info: WorkflowInfo):
        request = mock_workflow_info.create_workflow_request(
            mock_workflow_info.lookup_workflow_definition("null"), json.loads('{"argument":"-g"}')
        )
        self.requests.append(request)
        assert request in self.requests

    def test_attach_file_to_request(
        self, mock_workflow_info: WorkflowInfo, mock_workflow_requests: List[WorkflowRequest]
    ):
        request = mock_workflow_requests[0]
        test_filename = "test.txt"
        test_content = b"testing file additions"
        file = mock_workflow_info.save_file(
            request=request, filename=test_filename, content=test_content
        )
        mock_workflow_info.session.add.assert_called_with(file)
        assert mock_workflow_info.session.flush.call_count == 1

    def test_prepare_files_for_condor(self, mock_workflow_service: WorkflowService):
        files = [
            WorkflowRequestFile(
                workflow_request_id=1,
                filename="null.condor",
                content=b"executable = null\n"
                + b"arguments = {{arguments}}\n"
                + b"error = null.err\n"
                + b"log = condor.log\n\n\n"
                + b"queue",
            ),
            WorkflowRequestFile(
                workflow_request_id=1, filename="null.dag", content=b"JOB null null.condor"
            ),
            WorkflowRequestFile(
                workflow_request_id=1, filename="null.sh", content=b"#!/bin/sh\n\n" + b"null $*"
            ),
        ]

        path = mock_workflow_service._prepare_files_for_condor(files)

        assert os.path.exists(path) is True

    def test_execute_prepared(self):
        pass

    def test_get_log_file(self):
        pass

    def execute(self):
        pass
